
Vn.Catalog = new Class
({
	Extends: Vn.Module

	,realmColor: null

	,activate: function ()
	{
		this.popup = new Htk.Popup ();
		this.popup.setChildNode (this.$('lots-popup'));
	}

	,onRealmsReload: function (model, status)
	{
		if (status != Db.Model.Status.READY)
			return;

		this.onRealmChange ();
		this.onTypeChange ();
	}
	
	,onRealmChange: function ()
	{
		var color;
		var realms = this.$('realms-model');
		var row = realms.search ('id', this.$('realm').value);

		if (row != -1)
			this.realmColor = '#'+ realms.get (row, 'color');
		else
			this.realmColor = null;
	}

	,onTypesReload: function (model, status)
	{
		if (status == Db.Model.Status.READY)
			this.refreshTitle ();
	}

	,onTypeChange: function ()
	{
		if (Vn.isMobile ())
			this.hideMenu ();
		
		this.refreshTitle ();

		var color = null;
		
		if (this.$('type').value)
			color = this.realmColor;

		this.$('header').style.backgroundColor = color;

		var itemsGrid = this.$('items-grid').getNode ();
		var thead = itemsGrid.getElementsByTagName ('thead')[0];
		var tr = thead.getElementsByTagName ('tr')[0];
		tr.style.backgroundColor = color;
	}
	
	,refreshTitle: function (title)
	{
		var title = null;
		
		if (this.$('type').value)
		{
			var types = this.$('types-model');
			var row = types.search ('tipo_id', this.$('type').value);

			if (row != -1)
				title = types.get (row, 'Tipo');
		}
		else if (this.$('search-entry').value)
			title = _('SearchResults');

		if (title)
			Vn.Node.setText (this.$('title'), title);
	}

	,onSearch: function (event)
	{
		var searchTags = this.$('search-entry').value;
		searchTags = searchTags != '' ? searchTags : null;

		var batch = this.$('batch');
		batch.block ();

		this.$('search').value = searchTags;

		if (searchTags)
		{
			this.$('type').value = null;
			this.$('realm').value = null;
		}

		batch.unblock ();
		batch.changed ();
	}

	,onMenuClick: function (event)
	{
		event.stopPropagation ();
	}

	,onShowMenuClick: function (event)
	{
		event.stopPropagation ();
		this.gui.showBackground ();
		this.$('menu').style.display = 'block';
		this.hideMenuCallback = this.hideMenu.bind (this);
		document.addEventListener ('click', this.hideMenuCallback);
	}

	,hideMenu: function ()
	{
		this.gui.hideBackground ();
		this.$('menu').style.display = 'none';
		document.removeEventListener ('click', this.hideMenuCallback);
		this.hideMenuCallback = null;
	}
	
	,onBasketClick: function ()
	{
		this.hash.set ({'form': 'ecomerce/basket'});
	}
	
	,featuresRender: function (renderer, form)
	{
		renderer.value = form.get ('Medida') +' '+ form.get ('Categoria') +' '+ form.get ('Color');
	}
	
	,typeRenderer: function (column, form)
	{
		column.href = this.hash.make ({'type': form.get ('tipo_id')}, true);
	}
	
	,amountRender: function (renderer, form)
	{
		var amount = form.get ('amount');
		var grouping = form.get ('grouping');

		if (amount && grouping)
			renderer.value = amount / grouping;
		else
			renderer.value = null;
	}

	,amountChanged: function (renderer, row, newValue)
	{
		var model = this.$('items-model');
		model.set (row, 'amount', newValue * model.get (row, 'grouping'));
	}

	,catRenderer: function (column, form)
	{
		switch (form.get ('Categoria'))
		{
			case 'A2':
				column.td.className = 'second-category';
				break;
			case 'B1':
				column.td.className = 'third-category';
				break;
			default:
				column.td.className = '';
		}
	}
	
	,onAddItemClick: function  (column, value, row, button)
	{
		var itemId = this.$('items-model').get (row, 'item_id')
	
		var batch = this.$('lots-batch');
		batch.addValue ('item', itemId);
		batch.changed ();

		this.itemId = itemId;
		this.onEraseClick ();
		this.popup.show (button);
	}
	
	,onAddLotClick: function (column, value, row, button)
	{
		var model = this.$('item-lots');
		
		var grouping = model.get (row, 'grouping');
		var warehouse = model.get (row, 'warehouse_id');
		var rate = model.get (row, 'rate');
		
		for (var i = 0; i < this.items.length; i++)
		{
			var t = this.items[i];
			if (t.warehouse == warehouse && t.rate == rate)
				break;
		}
		
		if (i >= this.items.length)
			this.items.push ({
				warehouse: warehouse,
				rate: rate,
				amount: grouping
			});
		else
			t.amount += grouping;

		this.$('amount').value += grouping;
	}
	
	,onEraseClick: function ()
	{
		this.$('amount').value = 0;
		this.items = [];
	}
	
	,onConfirmClick: function ()
	{
		if (this.items.length > 0)
		{
			var sql = '';
			var batch = new Sql.Batch ();
			var query = new Sql.String ({query: 'CALL order_row_new (#warehouse, #item, #rate, #amount);'});
	
			for (var i = 0; i < this.items.length; i++)
			{
				var t = this.items[i];
				batch.addValue ('warehouse', t.warehouse);
				batch.addValue ('item', this.itemId);
				batch.addValue ('rate', t.rate);
				batch.addValue ('amount', t.amount);
				sql += query.render (batch);
			}

			this.conn.execQuery (sql);
		}

		this.popup.hide ();
	}
});

Htk.Realm = new Class
({
	Extends: Htk.Field
	,Tag: 'htk-realm'
	,Properties:
	{
		 model:
		 {
			type: Db.Model
			,set: function (x)
			{
				x.indexColumn ('id');
				this.link ({_model: x}, {'status-changed': this.onModelReload});
				this.onModelReload (x);
			}
			,get: function ()
			{
				return this._model;
			}
		}
	}

	,_model: null
	,selectedImg: null

	,initialize: function (props)
	{
		this.parent (props);
		this.createElement ('div');
		this.node.className = 'htk-realm';
	}

	,onModelReload: function (model)
	{
		Vn.Node.removeChilds (this.node);
	
		if (model.status != Db.Model.Status.READY)
			return;

		var width = parseInt (100 / model.numRows) +'%';

		for (var i = 0; i < model.numRows; i++)
		{
			var id = model.get (i, 'id');
		
			var img = document.createElement ('img');
//			img.src = Vn.Config['image_dir'] +'/family/full/'+ id +'.png';
			img.src = 'image/family/'+ id +'.svg';
			img.title = model.get (i, 'reino');
			img.alt = img.title;
			img.style.width = width;
			img.style.backgroundColor = '#'+ model.get ('color');
			img.addEventListener ('click', this.clicked.bind (this, img, id));
			this.node.appendChild (img);
			
			if (id == this.value)
				this.focusImage (img);
		}
	}
	
	,focusImage: function (img)
	{
		if (this.selectedImg)
			this.selectedImg.className = '';

		img.className = 'selected';
		this.selectedImg = img;
	}

	,clicked: function (img, value)
	{
		this.focusImage (img);
	
		if (value != this.value)
			this.valueChanged (value);
	}
	
	,putValue: function (value)
	{
		var row = this._model.search ('id', value);

		if (row != -1)
			this.focusImage (this.node.childNodes[row]);
	}
});

